from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field


T = TypeVar("T", bound="CreateMonitorTargetRequest")


@_attrs_define
class CreateMonitorTargetRequest:
    """
    Attributes:
        name (str):
        comment (str):
        repository_url (str):
        monitor_path (str):
        network_name (str):
    """

    name: str
    comment: str
    repository_url: str
    monitor_path: str
    network_name: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        comment = self.comment
        repository_url = self.repository_url
        monitor_path = self.monitor_path
        network_name = self.network_name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "comment": comment,
                "repository_url": repository_url,
                "monitor_path": monitor_path,
                "network_name": network_name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        comment = d.pop("comment")

        repository_url = d.pop("repository_url")

        monitor_path = d.pop("monitor_path")

        network_name = d.pop("network_name")

        create_monitor_target_request = cls(
            name=name,
            comment=comment,
            repository_url=repository_url,
            monitor_path=monitor_path,
            network_name=network_name,
        )

        create_monitor_target_request.additional_properties = d
        return create_monitor_target_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from typing import cast, Union
from ...models.body_upload_snapshot_organization_name_api_v1_uploadsnapshot_post import (
    BodyUploadSnapshotOrganizationNameApiV1UploadsnapshotPost,
)
from typing import Union
from ...models.base_error_response import BaseErrorResponse
from ...models.validation_error_response import ValidationErrorResponse
from ...models.challenge_response import ChallengeResponse
from ...types import UNSET, Unset
from typing import Dict
from ...models.upload_snapshot_response import UploadSnapshotResponse
from typing import cast


def _get_kwargs(
    organization_name: str,
    *,
    multipart_data: BodyUploadSnapshotOrganizationNameApiV1UploadsnapshotPost,
    network: Union[None, Unset, str] = UNSET,
    role: Union[None, Unset, str] = UNSET,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}
    json_network: Union[None, Unset, str]
    if isinstance(network, Unset):
        json_network = UNSET

    else:
        json_network = network

    params["network"] = json_network

    json_role: Union[None, Unset, str]
    if isinstance(role, Unset):
        json_role = UNSET

    else:
        json_role = role

    params["role"] = json_role

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    multipart_multipart_data = multipart_data.to_multipart()

    return {
        "method": "post",
        "url": "/{organization_name}/api/v1/uploadsnapshot/".format(
            organization_name=organization_name,
        ),
        "files": multipart_multipart_data,
        "params": params,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[
    Union[
        Any,
        BaseErrorResponse,
        ChallengeResponse,
        UploadSnapshotResponse,
        ValidationErrorResponse,
    ]
]:
    if response.status_code == HTTPStatus.OK:
        response_200 = UploadSnapshotResponse.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        response_422 = ValidationErrorResponse.from_dict(response.json())

        return response_422
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = ChallengeResponse.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = BaseErrorResponse.from_dict(response.json())

        return response_404
    if response.status_code == HTTPStatus.REQUEST_ENTITY_TOO_LARGE:
        response_413 = cast(Any, None)
        return response_413
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[
    Union[
        Any,
        BaseErrorResponse,
        ChallengeResponse,
        UploadSnapshotResponse,
        ValidationErrorResponse,
    ]
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    organization_name: str,
    *,
    client: AuthenticatedClient,
    multipart_data: BodyUploadSnapshotOrganizationNameApiV1UploadsnapshotPost,
    network: Union[None, Unset, str] = UNSET,
    role: Union[None, Unset, str] = UNSET,
) -> Response[
    Union[
        Any,
        BaseErrorResponse,
        ChallengeResponse,
        UploadSnapshotResponse,
        ValidationErrorResponse,
    ]
]:
    """Upload a network snapshot.

    Args:
        organization_name (str):
        network (Union[None, Unset, str]):
        role (Union[None, Unset, str]):
        multipart_data (BodyUploadSnapshotOrganizationNameApiV1UploadsnapshotPost):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, BaseErrorResponse, ChallengeResponse, UploadSnapshotResponse, ValidationErrorResponse]]
    """

    kwargs = _get_kwargs(
        organization_name=organization_name,
        multipart_data=multipart_data,
        network=network,
        role=role,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    organization_name: str,
    *,
    client: AuthenticatedClient,
    multipart_data: BodyUploadSnapshotOrganizationNameApiV1UploadsnapshotPost,
    network: Union[None, Unset, str] = UNSET,
    role: Union[None, Unset, str] = UNSET,
) -> Optional[
    Union[
        Any,
        BaseErrorResponse,
        ChallengeResponse,
        UploadSnapshotResponse,
        ValidationErrorResponse,
    ]
]:
    """Upload a network snapshot.

    Args:
        organization_name (str):
        network (Union[None, Unset, str]):
        role (Union[None, Unset, str]):
        multipart_data (BodyUploadSnapshotOrganizationNameApiV1UploadsnapshotPost):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, BaseErrorResponse, ChallengeResponse, UploadSnapshotResponse, ValidationErrorResponse]
    """

    return sync_detailed(
        organization_name=organization_name,
        client=client,
        multipart_data=multipart_data,
        network=network,
        role=role,
    ).parsed


async def asyncio_detailed(
    organization_name: str,
    *,
    client: AuthenticatedClient,
    multipart_data: BodyUploadSnapshotOrganizationNameApiV1UploadsnapshotPost,
    network: Union[None, Unset, str] = UNSET,
    role: Union[None, Unset, str] = UNSET,
) -> Response[
    Union[
        Any,
        BaseErrorResponse,
        ChallengeResponse,
        UploadSnapshotResponse,
        ValidationErrorResponse,
    ]
]:
    """Upload a network snapshot.

    Args:
        organization_name (str):
        network (Union[None, Unset, str]):
        role (Union[None, Unset, str]):
        multipart_data (BodyUploadSnapshotOrganizationNameApiV1UploadsnapshotPost):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, BaseErrorResponse, ChallengeResponse, UploadSnapshotResponse, ValidationErrorResponse]]
    """

    kwargs = _get_kwargs(
        organization_name=organization_name,
        multipart_data=multipart_data,
        network=network,
        role=role,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    organization_name: str,
    *,
    client: AuthenticatedClient,
    multipart_data: BodyUploadSnapshotOrganizationNameApiV1UploadsnapshotPost,
    network: Union[None, Unset, str] = UNSET,
    role: Union[None, Unset, str] = UNSET,
) -> Optional[
    Union[
        Any,
        BaseErrorResponse,
        ChallengeResponse,
        UploadSnapshotResponse,
        ValidationErrorResponse,
    ]
]:
    """Upload a network snapshot.

    Args:
        organization_name (str):
        network (Union[None, Unset, str]):
        role (Union[None, Unset, str]):
        multipart_data (BodyUploadSnapshotOrganizationNameApiV1UploadsnapshotPost):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, BaseErrorResponse, ChallengeResponse, UploadSnapshotResponse, ValidationErrorResponse]
    """

    return (
        await asyncio_detailed(
            organization_name=organization_name,
            client=client,
            multipart_data=multipart_data,
            network=network,
            role=role,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field


from typing import cast, Union


T = TypeVar("T", bound="ConsoleRequestOptions")


@_attrs_define
class ConsoleRequestOptions:
    """Computed information about the user's local terminal.

    Attributes:
        color_system (Union[None, str]):
        no_color (bool):
        width (int):
        height (int):
        is_jupyter (bool):
        is_terminal (bool):
        is_interactive (bool):
        legacy_windows (bool):
    """

    color_system: Union[None, str]
    no_color: bool
    width: int
    height: int
    is_jupyter: bool
    is_terminal: bool
    is_interactive: bool
    legacy_windows: bool
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        color_system: Union[None, str]

        color_system = self.color_system

        no_color = self.no_color
        width = self.width
        height = self.height
        is_jupyter = self.is_jupyter
        is_terminal = self.is_terminal
        is_interactive = self.is_interactive
        legacy_windows = self.legacy_windows

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "color_system": color_system,
                "no_color": no_color,
                "width": width,
                "height": height,
                "is_jupyter": is_jupyter,
                "is_terminal": is_terminal,
                "is_interactive": is_interactive,
                "legacy_windows": legacy_windows,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_color_system(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        color_system = _parse_color_system(d.pop("color_system"))

        no_color = d.pop("no_color")

        width = d.pop("width")

        height = d.pop("height")

        is_jupyter = d.pop("is_jupyter")

        is_terminal = d.pop("is_terminal")

        is_interactive = d.pop("is_interactive")

        legacy_windows = d.pop("legacy_windows")

        console_request_options = cls(
            color_system=color_system,
            no_color=no_color,
            width=width,
            height=height,
            is_jupyter=is_jupyter,
            is_terminal=is_terminal,
            is_interactive=is_interactive,
            legacy_windows=legacy_windows,
        )

        console_request_options.additional_properties = d
        return console_request_options

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field


from typing import Dict

if TYPE_CHECKING:
    from ..models.flags_response_environment import FlagsResponseEnvironment
    from ..models.flags_response_flags import FlagsResponseFlags


T = TypeVar("T", bound="FlagsResponse")


@_attrs_define
class FlagsResponse:
    """
    Attributes:
        etag (str):
        environment (FlagsResponseEnvironment):
        flags (FlagsResponseFlags):
    """

    etag: str
    environment: "FlagsResponseEnvironment"
    flags: "FlagsResponseFlags"
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        etag = self.etag
        environment = self.environment.to_dict()

        flags = self.flags.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "etag": etag,
                "environment": environment,
                "flags": flags,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.flags_response_environment import FlagsResponseEnvironment
        from ..models.flags_response_flags import FlagsResponseFlags

        d = src_dict.copy()
        etag = d.pop("etag")

        environment = FlagsResponseEnvironment.from_dict(d.pop("environment"))

        flags = FlagsResponseFlags.from_dict(d.pop("flags"))

        flags_response = cls(
            etag=etag,
            environment=environment,
            flags=flags,
        )

        flags_response.additional_properties = d
        return flags_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
